@@ -1,7 +1,7 @@
 (********************************************************************)
 (*                                                                  *)
 (*  The Why3 Verification Platform   /   The Why3 Development Team  *)
-(*  Copyright 2010-2018   --   Inria - CNRS - Paris-Sud University  *)
+(*  Copyright 2010-2019   --   Inria - CNRS - Paris-Sud University  *)
 (*                                                                  *)
 (*  This software is distributed under the terms of the GNU Lesser  *)
 (*  General Public License version 2.1, with the special exception  *)
@@ -171,6 +171,9 @@ val t_open_branch : term_branch -> pattern * term
 val t_open_quant : term_quant -> vsymbol list * trigger * term
 
 val t_open_bound_with : term -> term_bound -> term
+(** [t_open_bound_with t tb] opens the binding [tb] and immediately
+    replaces the corresponding bound variable with [t] *)
+
 val t_clone_bound_id : term_bound -> preid
 
 (** open bindings with optimized closing callbacks *)
