@@ -1,7 +1,7 @@
 (********************************************************************)
 (*                                                                  *)
 (*  The Why3 Verification Platform   /   The Why3 Development Team  *)
-(*  Copyright 2010-2018   --   Inria - CNRS - Paris-Sud University  *)
+(*  Copyright 2010-2019   --   Inria - CNRS - Paris-Sud University  *)
 (*                                                                  *)
 (*  This software is distributed under the terms of the GNU Lesser  *)
 (*  General Public License version 2.1, with the special exception  *)
@@ -145,18 +145,15 @@ module Print = struct
             List.exists (rs_equal rs) its.itd_constructors in
           List.exists is_constructor its
       | _ -> false in
-    match query_syntax info.info_convert rs.rs_name,
-          query_syntax info.info_syn rs.rs_name, pvl with
-    | Some s, _, [{e_node = Econst _}] ->
-        syntax_arguments s print_constant fmt pvl
-    | _, Some s, _ (* when is_local_id info rs.rs_name  *)->
+    match query_syntax info.info_syn rs.rs_name, pvl with
+    | Some s, _ (* when is_local_id info rs.rs_name  *)->
         syntax_arguments s (print_expr ~paren:true info) fmt pvl;
-    | _, None, tl when is_rs_tuple rs ->
+    | None, tl when is_rs_tuple rs ->
         fprintf fmt "@[(%a)@]"
           (print_list comma (print_expr info)) tl
-    | _, None, [t1] when isfield ->
+    | None, [t1] when isfield ->
         fprintf fmt "%a.%a" (print_expr info) t1 (print_lident info) rs.rs_name
-    | _, None, tl when isconstructor () -> let pjl = get_record info rs in
+    | None, tl when isconstructor () -> let pjl = get_record info rs in
         begin match pjl, tl with
           | [], [] ->
               (print_uident info) fmt rs.rs_name
@@ -171,9 +168,9 @@ module Print = struct
               fprintf fmt "@[<hov 2>{ @[%a@] }@]"
                 (print_list2 semi equal (print_rs info) (print_expr info))
                 (pjl, tl) end
-    | _, None, [] ->
+    | None, [] ->
         (print_lident info) fmt rs.rs_name
-    | _, _, tl ->
+    | _, tl ->
         fprintf fmt "@[<hov 2>%a %a@]" (print_lident info) rs.rs_name
           (print_apply_args info) (tl, rs.rs_cty.cty_args)
 
@@ -201,11 +198,11 @@ module Print = struct
     | Lvar (pv, e) ->
         fprintf fmt "@[<hov 2>val %a =@ %a@]" (print_lident info) (pv_name pv)
           (print_expr info) e
-    | Lsym (rs, _, [], ef) ->
+    | Lsym (rs, _, _, [], ef) ->
         (* TODO? zero-arguments functions as Lvar in compile.Translate *)
         fprintf fmt "@[<hov 2>val %a =@ @[%a@]@]"
           (print_lident info) rs.rs_name (print_expr info) ef;
-    | Lsym (rs, _, args, ef) ->
+    | Lsym (rs, _, _, args, ef) ->
         fprintf fmt "@[<hov 2>fun %a @[%a@] =@ @[%a@]@]"
           (print_lident info) rs.rs_name
           (print_list space (print_vs_arg info)) args
@@ -221,7 +218,7 @@ module Print = struct
                 (print_fun_type_args info) (args, s, res, e);
               forget_vars args in
         print_list_next newline print_one fmt rdef;
-    | Lany (rs, _, _) ->
+    | Lany (rs, _, _, _) ->
         check_val_in_drv info rs
 
   and print_expr ?(paren=false) info fmt e =
@@ -243,8 +240,6 @@ module Print = struct
         forget_let_defn let_def
     | Eabsurd ->
         fprintf fmt (protect_on paren "assert false (* absurd *)")
-    | Ehole -> ()
-    | Eany _ -> assert false
     | Eapp (rs, []) when rs_equal rs rs_true ->
         fprintf fmt "true"
     | Eapp (rs, []) when rs_equal rs rs_false ->
@@ -252,12 +247,8 @@ module Print = struct
     | Eapp (rs, [])  -> (* avoids parenthesis around values *)
         fprintf fmt "%a" (print_apply info rs) []
     | Eapp (rs, pvl) ->
-        begin match query_syntax info.info_convert rs.rs_name, pvl with
-          | Some s, [{e_node = Econst _}] ->
-              syntax_arguments s print_constant fmt pvl
-          | _ ->
-              fprintf fmt (protect_on paren "%a")
-                (print_apply info rs) pvl end
+       fprintf fmt (protect_on paren "%a")
+               (print_apply info rs) pvl
     | Ematch (e1, [p, e2], []) ->
         fprintf fmt (protect_on paren "let %a =@ %a in@ %a")
           (print_pat info) p (print_expr info) e1 (print_expr info) e2
@@ -275,10 +266,10 @@ module Print = struct
           (print_expr info) e (print_list_next newline (print_branch info)) bl
           (print_list_next newline (print_xbranch true info)) xl
     | Eassign al ->
-        let assign fmt (rho, rs, pv) =
+        let assign fmt (rho, rs, e) =
           fprintf fmt "@[<hov 2>%a.%a <-@ %a@]"
             (print_lident info) (pv_name rho) (print_lident info) rs.rs_name
-            (print_lident info) (pv_name pv) in
+            (print_expr info) e in
         begin match al with
           | [] -> assert false | [a] -> assign fmt a
           | al -> fprintf fmt "@[(%a)]" (print_list semi assign) al end
@@ -432,6 +423,10 @@ module Print = struct
   let rec print_decl info fmt = function
     | Dlet ldef ->
         print_let_def info fmt ldef
+    | Dval ({pv_vs}, _) ->
+        let loc = pv_vs.vs_name.id_loc in
+        Loc.errorm ?loc "cannot extract top-level undefined constant %a@."
+          (print_lident info) pv_vs.vs_name
     | Dtype dl ->
         print_list newline (print_type_decl info) fmt dl
     | Dexn (xs, None) ->
