@@ -1,7 +1,7 @@
 (********************************************************************)
 (*                                                                  *)
 (*  The Why3 Verification Platform   /   The Why3 Development Team  *)
-(*  Copyright 2010-2018   --   Inria - CNRS - Paris-Sud University  *)
+(*  Copyright 2010-2019   --   Inria - CNRS - Paris-Sud University  *)
 (*                                                                  *)
 (*  This software is distributed under the terms of the GNU Lesser  *)
 (*  General Public License version 2.1, with the special exception  *)
@@ -234,6 +234,13 @@ module Translate = struct
     | ML.Tapp (_, tyl) | ML.Ttuple tyl ->
         List.fold_left add_tvar acc tyl
 
+  let new_svar args res svar =
+    let new_svar =
+      let args' = List.map (fun (_, ty, _) -> ty) args in
+      let svar  = List.fold_left add_tvar Stv.empty args' in
+      add_tvar svar res in
+    Stv.diff new_svar svar
+
   (* expressions *)
   let rec expr info svar mask ({e_effect = eff; e_attrs = attrs} as e) =
     assert (not (e_ghost e));
@@ -280,20 +287,25 @@ module Translate = struct
         ML.e_let ld (expr info svar mask e2) (ML.I e.e_ity) mask eff attrs
     | Elet (LDsym (rs, _), ein) when rs_ghost rs ->
         expr info svar mask ein
+    | Elet (LDsym (_, {c_node = Cpur (_, _); _}), _) ->
+        assert false (* necessarily handled above *)
     | Elet (LDsym (rs, {c_node = Cfun ef; c_cty = cty}), ein) ->
         Debug.dprintf debug_compile "compiling local function definition %s@."
           rs.rs_name.id_string;
         let args = params cty.cty_args in
         let res = mlty_of_ity cty.cty_mask cty.cty_result in
-        let ld = ML.sym_defn rs res args (expr info svar cty.cty_mask ef) in
+        let new_svar = new_svar args res svar in
+        let ld = ML.sym_defn rs new_svar res args (expr info svar cty.cty_mask ef) in
         ML.e_let ld (expr info svar mask ein) (ML.I e.e_ity) mask eff attrs
     | Elet (LDsym (rs, {c_node = Capp (rs_app, pvl); c_cty = cty}), ein)
       when isconstructor info rs_app -> (* partial application of constructor *)
         let eta_app = mk_eta_expansion rs_app pvl cty in
         let mk_func pv f = ity_func pv.pv_ity f in
         let func = List.fold_right mk_func cty.cty_args cty.cty_result in
+        let args = params cty.cty_args in
         let res = mlty_of_ity cty.cty_mask func in
-        let ld = ML.sym_defn rs res [] eta_app in
+        let new_svar = new_svar args res svar in
+        let ld = ML.sym_defn rs new_svar res [] eta_app in
         let ein = expr info svar mask ein in
         ML.e_let ld ein (ML.I e.e_ity) mask eff attrs
     | Elet (LDsym (rsf, {c_node = Capp (rs_app, pvl); c_cty = cty}), ein) ->
@@ -305,10 +317,14 @@ module Translate = struct
         let pvl = app pvl rs_app.rs_cty.cty_args (fun x -> x) in
         let rs_app = Hrs.find_def ht_rs rs_app rs_app in
         let eapp = ML.e_app rs_app pvl (ML.C cty) cmk ceff Sattr.empty in
+        let args = params cty.cty_args in
         let res = mlty_of_ity cty.cty_mask cty.cty_result in
-        let ld = ML.sym_defn rsf res (params cty.cty_args) eapp in
+        let new_svar = new_svar args res svar in
+        let ld = ML.sym_defn rsf new_svar res args eapp in
         let ein = expr info svar mask ein in
         ML.e_let ld ein (ML.I e.e_ity) mask eff attrs
+    | Elet (LDsym (_, {c_node = Cany; _}), _) -> let loc = e.e_loc in
+        Loc.errorm ?loc "This expression cannot be extracted"
     | Elet (LDrec rdefl, ein) ->
         let rdefl = filter_out_ghost_rdef rdefl in
         List.iter
@@ -318,11 +334,7 @@ module Translate = struct
           | { rec_sym = rs1; rec_fun = {c_node = Cfun ef; c_cty = cty} } ->
               let res = mlty_of_ity rs1.rs_cty.cty_mask rs1.rs_cty.cty_result in
               let args = params cty.cty_args in
-              let new_svar =
-                let args' = List.map (fun (_, ty, _) -> ty) args in
-                let svar  = List.fold_left add_tvar Stv.empty args' in
-                add_tvar svar res in
-              let new_svar = Stv.diff svar new_svar in
+              let new_svar = new_svar args res svar in
               let ef = expr info (Stv.union svar new_svar) ef.e_mask ef in
               { ML.rec_sym  = rs1; ML.rec_args = args; ML.rec_exp  = ef;
                 ML.rec_res  = res; ML.rec_svar = new_svar; }
@@ -347,15 +359,18 @@ module Translate = struct
         (* partial application of constructors *)
         mk_eta_expansion rs pvl cty
     | Eexec ({c_node = Capp (rs, pvl); c_cty = cty}, _) ->
-        Debug.dprintf debug_compile "compiling total application of %s@."
+        Debug.dprintf debug_compile "compiling application of %s@."
           rs.rs_name.id_string;
+        Debug.dprintf debug_compile "pvl: %d@." (List.length pvl);
         Debug.dprintf debug_compile "cty_args: %d@." (List.length cty.cty_args);
         let rs = Hrs.find_def ht_rs rs rs in
         let add_unit = function [] -> [ML.e_unit] | args -> args in
         let id_f = fun x -> x in
-        let f_zero = match rs.rs_logic with RLnone ->
-          Debug.dprintf debug_compile "it is a RLnone@."; add_unit
-                                          | _ -> id_f in
+        let f_zero = match rs.rs_logic with
+          | RLnone when cty.cty_args = []  ->
+              Debug.dprintf debug_compile "it is a fully applied RLnone@.";
+              add_unit
+          | _ -> id_f in
         let pvl = app pvl rs.rs_cty.cty_args f_zero in
         begin match pvl with
           | [pv_expr] when is_optimizable_record_rs info rs -> pv_expr
@@ -369,8 +384,10 @@ module Translate = struct
         Debug.dprintf debug_compile "compiling a lambda expression@.";
         let ef = expr info svar e.e_mask ef in
         ML.e_fun (params cty.cty_args) ef (ML.I e.e_ity) mask eff attrs
-    | Eexec ({c_node = Cany}, _) ->
-        ML.mk_hole
+    | Eexec ({c_node = Cpur (_, _); _ }, _) ->
+        assert false (* necessarily ghost *)
+    | Eexec ({c_node = Cany}, _) -> let loc = e.e_loc in
+        Loc.errorm ?loc "This expression cannot be extracted"
     | Eabsurd ->
         ML.e_absurd (ML.I e.e_ity) mask eff attrs
     | Eassert _ ->
@@ -408,6 +425,8 @@ module Translate = struct
     | Eassign al ->
         let rm_ghost (_, rs, _) = not (rs_ghost rs) in
         let al = List.filter rm_ghost al in
+        let e_of_var pv = ML.e_var pv (ML.I pv.pv_ity) MaskVisible eff attrs in
+        let al = List.map (fun (pv1, rs, pv2) -> (pv1, rs, e_of_var pv2)) al in
         ML.e_assign al (ML.I e.e_ity) mask eff attrs
     | Ematch (e1, bl, xl) when e_ghost e1 ->
         assert (Mxs.is_empty xl); (* Expr ensures this for the time being *)
@@ -438,8 +457,6 @@ module Translate = struct
           let ty = if ity_equal xs.xs_ity ity_unit then None
             else Some (mlty_of_ity xs.xs_mask xs.xs_ity) in
         ML.mk_expr (ML.Eexn (xs, ty, e1)) (ML.I e.e_ity) mask eff attrs
-    | Elet (LDsym (_, {c_node=(Cany|Cpur (_, _)); _ }), _)
-    | Eexec ({c_node=Cpur (_, _); _ }, _) -> ML.mk_hole
 
   and ebranch info svar mask ({pp_pat = p; pp_mask = m}, e) =
     (* if the [case] expression is not ghost but there is (at least) one ghost
@@ -505,11 +522,11 @@ module Translate = struct
         if eff_pure e.e_effect then []
         else let unit_ = pv (* create_pvsymbol (id_fresh "_") ity_unit *) in
           [ML.Dlet (ML.Lvar (unit_, expr info Stv.empty MaskGhost e))]
-    | PDlet (LDvar (pv,  {e_node = Eexec ({c_node = Cany}, cty)})) ->
-        Debug.dprintf debug_compile "compiling undifined constant %a@"
+    | PDlet (LDvar (pv, {e_node = Eexec ({c_node = Cany}, cty)})) ->
+        Debug.dprintf debug_compile "compiling undefined constant %a@"
           print_pv pv;
         let ty = mlty_of_ity cty.cty_mask cty.cty_result in
-        [ML.Dlet (ML.Lvar (pv, ML.e_any ty cty))]
+        [ML.Dval (pv, ty)]
     | PDlet (LDvar (pv, e)) ->
         Debug.dprintf debug_compile "compiling top-level symbol %a@."
           print_pv pv;
@@ -519,11 +536,14 @@ module Translate = struct
     | PDlet (LDsym ({rs_cty = cty} as rs, {c_node = Cany})) ->
         let args = params cty.cty_args in
         let res = mlty_of_ity cty.cty_mask cty.cty_result in
-        [ML.Dlet (ML.Lany (rs, res, args))]
+        let new_svar = new_svar args res Stv.empty in
+        [ML.Dlet (ML.Lany (rs, new_svar, res, args))]
     | PDlet (LDsym ({rs_cty = cty} as rs, {c_node = Cfun e}))
       when is_val e.e_node -> (* zero argument functions *)
         let res = mlty_of_ity cty.cty_mask cty.cty_result in
-        [ML.Dlet (ML.Lany (rs, res, []))]
+        let args = params cty.cty_args in
+        let new_svar = new_svar args res Stv.empty in
+        [ML.Dlet (ML.Lany (rs, new_svar, res, []))]
     | PDlet (LDsym ({rs_cty = cty; rs_logic} as rs, {c_node = Cfun e; c_cty}))
       when c_cty.cty_args = [] ->
         Debug.dprintf debug_compile "compiling zero-arguments function %a@."
@@ -535,22 +555,16 @@ module Translate = struct
           Debug.dprintf debug_compile "rlnone ici@."; [ML.mk_var_unit]
                                      | _ -> [] in
         let res = mlty_of_ity cty.cty_mask cty.cty_result in
-        let svar =
-          let args' = List.map (fun (_, ty, _) -> ty) args in
-          let svar  = List.fold_left add_tvar Stv.empty args' in
-          add_tvar svar res in
+        let svar = new_svar args res Stv.empty in
         let e = expr info svar cty.cty_mask e in
-        [ML.Dlet (ML.Lsym (rs, res, args, e))]
+        [ML.Dlet (ML.Lsym (rs, svar, res, args, e))]
     | PDlet (LDsym ({rs_cty = cty} as rs, {c_node = Cfun e})) ->
         Debug.dprintf debug_compile "compiling function %a@." Expr.print_rs rs;
         let args = params cty.cty_args in
         let res = mlty_of_ity cty.cty_mask cty.cty_result in
-        let svar =
-          let args' = List.map (fun (_, ty, _) -> ty) args in
-          let svar  = List.fold_left add_tvar Stv.empty args' in
-          add_tvar svar res in
+        let svar = new_svar args res Stv.empty in
         let e = expr info svar cty.cty_mask e in
-        [ML.Dlet (ML.Lsym (rs, res, args, e))]
+        [ML.Dlet (ML.Lsym (rs, svar, res, args, e))]
     | PDlet (LDrec rl) ->
         let rl = filter_out_ghost_rdef rl in
         List.iter (fun {rec_sym = rs1; rec_rsym = rs2} ->
@@ -609,11 +623,17 @@ module Transform = struct
 
   open Mltree
 
-  let no_reads_writes_conflict spv spv_mreg =
-    let is_not_write {pv_ity = ity} = match ity.ity_node with
-      | Ityreg rho -> not (Mreg.mem rho spv_mreg)
-      | _ -> true in
-    Spv.for_all is_not_write spv
+  let no_effect_conflict spv eff =
+    Spv.is_empty (pvs_affected eff.eff_writes spv) &&
+    Spv.is_empty (pvs_affected eff.eff_resets spv)
+
+  let rec can_inline ({e_effect = eff1} as e1) ({e_effect = eff2} as e2) =
+    match e2.e_node with
+    | Evar _ | Econst _ | Eapp _ | Eassign [_] -> true
+    | Elet (Lvar (_, {e_effect = eff1'}), e2') ->
+       no_effect_conflict eff1.eff_reads eff1'
+       && can_inline e1 e2'
+    | _ -> no_effect_conflict eff1.eff_reads eff2
 
   let mk_list_eb ebl f =
     let mk_acc e (e_acc, s_acc) =
@@ -626,10 +646,10 @@ module Transform = struct
     match e.e_node with
     | Evar pv -> begin try Mpv.find pv subst, Spv.singleton pv
         with Not_found -> e, Spv.empty end
-    | Elet (Lvar (pv, ({e_effect = eff1} as e1)), ({e_effect = eff2} as e2))
-      when Sattr.mem Expr.proxy_attr pv.pv_vs.vs_name.id_attrs &&
+    | Elet (Lvar (pv, ({e_effect = eff1} as e1)), e2)
+      when Sattr.mem proxy_attr pv.pv_vs.vs_name.id_attrs &&
            eff_pure eff1 &&
-           no_reads_writes_conflict eff1.eff_reads eff2.eff_writes ->
+           can_inline e1 e2 ->
         let e1, s1 = expr info subst e1 in
         let e2, s2 = add_subst pv e1 e2 in
         let s_union = Spv.union s1 s2 in
@@ -689,9 +709,15 @@ module Transform = struct
     | Eraise (exn, Some e) ->
         let e, spv = expr info subst e in
         mk (Eraise (exn, Some e)), spv
-    | Eassign _al ->
-        e, Spv.empty
-    | Econst _ | Eabsurd | Ehole | Eany _ -> e, Spv.empty
+    | Eassign al ->
+       let al, s =
+         List.fold_left
+           (fun (accl, spv) (pv,rs,e) ->
+             let e, s = expr info subst e in
+             ((pv, rs, e)::accl, Spv.union spv s))
+           ([], Spv.empty) al in
+       mk (Eassign (List.rev al)), s
+    | Econst _ | Eabsurd -> e, Spv.empty
     | Eignore e ->
         let e, spv = expr info subst e in
         mk (Eignore e), spv
@@ -706,9 +732,9 @@ module Transform = struct
         assert (not (Mpv.mem pv subst)); (* no capture *)
         let e, spv = expr info subst e in
         Lvar (pv, e), spv
-    | Lsym (rs, res, args, e) ->
+    | Lsym (rs, svar, res, args, e) ->
         let e, spv = expr info subst e in
-        Lsym (rs, res, args, e), spv
+        Lsym (rs, svar, res, args, e), spv
     | Lany _ as lany -> lany, Mpv.empty
     | Lrec rl ->
         let rdef, spv = mk_list_eb rl (rdef info subst) in
@@ -719,7 +745,7 @@ module Transform = struct
     { r with rec_exp = rec_exp }, spv
 
   let rec pdecl info = function
-    | Dtype _ | Dexn _ as d -> d
+    | Dtype _ | Dexn _ | Dval _ as d -> d
     | Dmodule (id, dl) ->
         let dl = List.map (pdecl info) dl in Dmodule (id, dl)
     | Dlet def ->
