@@ -1,7 +1,7 @@
 (********************************************************************)
 (*                                                                  *)
 (*  The Why3 Verification Platform   /   The Why3 Development Team  *)
-(*  Copyright 2010-2018   --   Inria - CNRS - Paris-Sud University  *)
+(*  Copyright 2010-2019   --   Inria - CNRS - Paris-Sud University  *)
 (*                                                                  *)
 (*  This software is distributed under the terms of the GNU Lesser  *)
 (*  General Public License version 2.1, with the special exception  *)
@@ -391,7 +391,7 @@ let exec_print _ args =
 
 let built_in_modules =
   [
-    ["bool"],"Bool", [],
+    ["why3"; "Bool"],"Bool", [],
     [ "True", eval_true ;
       "False", eval_false ;
     ] ;
@@ -505,14 +505,16 @@ let built_in_modules =
      "set", exec_matrix_set ;
      "copy", exec_matrix_copy ;
     ] ;
-    ["ref"],"Ref",
-    [], (* ? *)
+    ["why3"; "Ref"],"Ref", [],
     ["ref", exec_ref_make ;
-     Ident.op_prefix "!", exec_ref_get;
+     "mk ref", exec_ref_make ;
+     "contents", exec_ref_get ;
+    ] ;
+    ["ref"],"Ref", [],
+    [Ident.op_prefix "!", exec_ref_get;
      Ident.op_infix ":=", exec_ref_set;
     ] ;
-    ["debug"],"Debug",
-    [],
+    ["debug"],"Debug", [],
     ["print", exec_print ] ;
   ]
 
@@ -641,7 +643,7 @@ let rec interp_expr info (e:Mltree.expr) : value =
                      with Not_found -> info.get_decl rs in
           Debug.dprintf debug_interp "decl found@.";
           match decl with
-          | Dlet (Lsym (rs, _ty, vl, e)) ->
+          | Dlet (Lsym (rs, _, _ty, vl, e)) ->
              eval_call info vl e rs
           | Dlet(Lrec([{rec_args = vl; rec_exp = e;
                         rec_sym = rs; rec_res=_ty}])) ->
@@ -696,9 +698,9 @@ let rec interp_expr info (e:Mltree.expr) : value =
      end
   | Eassign l ->
      List.iter
-       (fun (pvs, rs, v) ->
+       (fun (pvs, rs, e) ->
          let fld = fd_of_rs rs in
-         let value = get v info in
+         let value = interp_expr info e in
          match get pvs info with
          | Vconstr(c, args) ->
             let rec aux cargs args =
@@ -759,7 +761,7 @@ let rec interp_expr info (e:Mltree.expr) : value =
      end
   | Elet (Lany _,_) -> Debug.dprintf debug_interp "unhandled Lany@.";
                        raise CannotReduce
-  | Elet ((Lsym(rs,_,_,_) as ld), e) ->
+  | Elet ((Lsym(rs,_,_,_,_) as ld), e) ->
      interp_expr (add_fundecl rs (Dlet ld) info) e
   | Elet ((Lrec rdl as ld), e) ->
      let info = List.fold_left
@@ -776,10 +778,6 @@ let rec interp_expr info (e:Mltree.expr) : value =
                          raise CannotReduce
   | Eabsurd -> Debug.dprintf debug_interp "Eabsurd@.";
                raise CannotReduce
-  | Ehole -> Debug.dprintf debug_interp "Ehole@.";
-             raise CannotReduce
-  | Eany _ -> Debug.dprintf debug_interp "Eany@.";
-             raise CannotReduce
   | Ematch (e, l, bl) ->
      Debug.dprintf debug_interp "Ematch@.";
      begin match interp_expr info e with
@@ -872,6 +870,6 @@ let interp env mm rs vars =
   if Debug.test_flag debug_flamegraph then ts := Unix.gettimeofday ();
   let decl = info.get_decl rs in
   match decl with
-  | Dlet (Lsym (_rs, _, _vl, expr)) ->
+  | Dlet (Lsym (_rs, _, _, _vl, expr)) ->
      interp_expr info expr
   | _ -> raise CannotReduce
