@@ -1,7 +1,7 @@
 (********************************************************************)
 (*                                                                  *)
 (*  The Why3 Verification Platform   /   The Why3 Development Team  *)
-(*  Copyright 2010-2018   --   Inria - CNRS - Paris-Sud University  *)
+(*  Copyright 2010-2019   --   Inria - CNRS - Paris-Sud University  *)
 (*                                                                  *)
 (*  This software is distributed under the terms of the GNU Lesser  *)
 (*  General Public License version 2.1, with the special exception  *)
@@ -71,6 +71,12 @@ let same_overload r1 r2 =
   | FixedRes t1, FixedRes t2 -> ity_equal t1 t2
   | _ -> false (* two NoOver's are not the same *)
 
+let ref_attr = Ident.create_attribute "mlw:reference_var"
+
+let has_ref s =
+  let has v = Sattr.mem ref_attr v.pv_vs.vs_name.id_attrs in
+  List.exists has s.rs_cty.cty_args
+
 exception IncompatibleNotation of string
 
 let merge_ps chk x vo vn =
@@ -87,12 +93,14 @@ let merge_ps chk x vo vn =
     (* but we can merge two compatible symbols *)
     | RS r1, RS r2 ->
         if rs_equal r1 r2 then vo else
+        if has_ref r1 || has_ref r2 then vn else
         if not (same_overload r1 r2) then vn else
         if ity_equal (fsty r1) (fsty r2) then vn else
         OO (Srs.add r2 (Srs.singleton r1))
     (* or add a compatible symbol to notation *)
     | OO s1, RS r2 ->
         if Srs.mem r2 s1 then vo else
+        if has_ref r2 then vn else
         let r1 = Srs.choose s1 in
         if not (same_overload r1 r2) then vn else
         let ty = fsty r2 in
@@ -398,6 +406,32 @@ let unit_module =
   let td = create_alias_decl (id_fresh "unit") [] ity_unit in
   close_module (List.fold_left add uc (create_type_decl [td]))
 
+let itd_ref =
+  let tv = create_tvsymbol (id_fresh "a") in
+  let pj = create_pvsymbol (id_fresh "contents") (ity_var tv) in
+  create_plain_record_decl ~priv:false ~mut:true (id_fresh "ref")
+                                            [tv] [true, pj] [] []
+
+let its_ref = itd_ref.itd_its
+let rs_ref_cons = List.hd itd_ref.itd_constructors
+let rs_ref_proj = List.hd itd_ref.itd_fields
+
+let ts_ref = its_ref.its_ts
+let ls_ref_cons = ls_of_rs rs_ref_cons
+let ls_ref_proj = ls_of_rs rs_ref_proj
+
+let rs_ref_ld, rs_ref =
+  let cty = rs_ref_cons.rs_cty in
+  let ityl = List.map (fun v -> v.pv_ity) cty.cty_args in
+  let_sym (id_fresh "ref") (c_app rs_ref_cons [] ityl cty.cty_result)
+
+let ref_module =
+  let add uc d = add_pdecl_raw ~warn:false uc d in
+  let uc = empty_module dummy_env (id_fresh "Ref") ["why3";"Ref"] in
+  let uc = List.fold_left add uc (create_type_decl [itd_ref]) in
+  let uc = use_export uc builtin_module (* needed for "=" *) in
+  close_module (add uc (create_let_decl rs_ref_ld))
+
 let create_module env ?(path=[]) n =
   let m = empty_module env n path in
   let m = use_export m builtin_module in
@@ -408,6 +442,8 @@ let create_module env ?(path=[]) n =
 let add_use uc d = Sid.fold (fun id uc ->
   if id_equal id ts_func.ts_name then
     use_export uc highord_module
+  else if id_equal id ts_ref.ts_name then
+    use_export uc ref_module
   else match is_ts_tuple_id id with
   | Some n -> use_export uc (tuple_module n)
   | None -> uc) (Mid.set_diff d.pd_syms uc.muc_known) uc
@@ -793,8 +829,8 @@ let rec clone_expr cl sm e = e_attr_copy e (match e.e_node with
       e_for v'
         (e_var (sm_find_pv sm f)) dir (e_var (sm_find_pv sm t))
         i' (clone_invl cl ism invl) (clone_expr cl ism e)
-  | Eraise (xs, e) ->
-      e_raise (sm_find_xs sm xs) (clone_expr cl sm e) (clone_ity cl e.e_ity)
+  | Eraise (xs, e1) ->
+      e_raise (sm_find_xs sm xs) (clone_expr cl sm e1) (clone_ity cl e.e_ity)
   | Eexn ({xs_name = id; xs_mask = mask; xs_ity = ity} as xs, e) ->
       let xs' = create_xsymbol (id_clone id) ~mask (clone_ity cl ity) in
       e_exn xs' (clone_expr cl (sm_save_xs sm xs xs') e)
@@ -1187,6 +1223,7 @@ end)
 let mlw_language_builtin =
   let builtin s =
     if s = unit_module.mod_theory.th_name.id_string then unit_module else
+    if s = ref_module.mod_theory.th_name.id_string then ref_module else
     if s = builtin_theory.th_name.id_string then builtin_module else
     if s = highord_theory.th_name.id_string then highord_module else
     if s = bool_theory.th_name.id_string then bool_module else
