@@ -1,7 +1,7 @@
 (********************************************************************)
 (*                                                                  *)
 (*  The Why3 Verification Platform   /   The Why3 Development Team  *)
-(*  Copyright 2010-2018   --   Inria - CNRS - Paris-Sud University  *)
+(*  Copyright 2010-2019   --   Inria - CNRS - Paris-Sud University  *)
 (*                                                                  *)
 (*  This software is distributed under the terms of the GNU Lesser  *)
 (*  General Public License version 2.1, with the special exception  *)
@@ -48,6 +48,8 @@ type overload =
 
 val overload_of_rs : rsymbol -> overload
 
+val ref_attr : Ident.attribute
+
 exception IncompatibleNotation of string
 
 (** {2 Module} *)
@@ -138,6 +140,15 @@ val bool_module : pmodule
 val unit_module : pmodule
 val highord_module : pmodule
 val tuple_module : int -> pmodule
+val ref_module : pmodule
+
+val its_ref     : itysymbol
+val ts_ref      : tysymbol
+val rs_ref      : rsymbol
+val rs_ref_cons : rsymbol
+val rs_ref_proj : rsymbol
+val ls_ref_cons : lsymbol
+val ls_ref_proj : lsymbol
 
 (** {2 WhyML language} *)
 
