@@ -1,7 +1,7 @@
 (********************************************************************)
 (*                                                                  *)
 (*  The Why3 Verification Platform   /   The Why3 Development Team  *)
-(*  Copyright 2010-2018   --   Inria - CNRS - Paris-Sud University  *)
+(*  Copyright 2010-2019   --   Inria - CNRS - Paris-Sud University  *)
 (*                                                                  *)
 (*  This software is distributed under the terms of the GNU Lesser  *)
 (*  General Public License version 2.1, with the special exception  *)
@@ -118,6 +118,8 @@ let rec ty_of_pty ns = function
   | PTtuple tyl ->
       let s = its_tuple (List.length tyl) in
       ty_app s.its_ts (List.map (ty_of_pty ns) tyl)
+  | PTref tyl ->
+      ty_app ts_ref (List.map (ty_of_pty ns) tyl)
   | PTarrow (ty1, ty2) ->
       ty_func (ty_of_pty ns ty1) (ty_of_pty ns ty2)
   | PTscope (q, ty) ->
@@ -144,12 +146,18 @@ let dty_of_opt ns = function
 
 (** Typing patterns, terms, and formulas *)
 
-let create_user_id {id_str = n; id_ats = attrs; id_loc = loc} =
+let create_user_prog_id {id_str = n; id_ats = attrs; id_loc = loc} =
   let get_attrs (attrs, loc) = function
     | ATstr attr -> Sattr.add attr attrs, loc | ATpos loc -> attrs, loc in
   let attrs, loc = List.fold_left get_attrs (Sattr.empty, loc) attrs in
   id_user ~attrs n loc
 
+let create_user_id id =
+  let id = create_user_prog_id id in
+  if Sattr.mem Pmodule.ref_attr id.pre_attrs then Loc.errorm ?loc:id.pre_loc
+    "reference markers are only admitted over program variables";
+  id
+
 let parse_record ~loc ns km get_val fl =
   let fl = List.map (fun (q,e) -> find_lsymbol_ns ns q, e) fl in
   let cs,pjl,flm = Loc.try2 ~loc parse_record km fl in
@@ -218,6 +226,48 @@ let mk_closure crcmap loc ls =
   let vl = Lists.mapi mk_v ls.ls_args in
   DTquant (DTlambda, vl, [], mk (DTapp (ls, List.map mk_t vl)))
 
+(* handle auto-dereference in logical terms *)
+
+let vs_dref vs = Sattr.mem Pmodule.ref_attr vs.vs_name.id_attrs
+
+let rec to_deref = function
+  | DTattr ({dt_node = DTvar _}, attrs)
+    when Sattr.mem Pmodule.ref_attr attrs -> true (* needed for DEpure *)
+  | DTattr (dt,_) | DTuloc (dt,_) | DTcast (dt,_) -> to_deref dt.dt_node
+  | DTgvar vs -> vs_dref vs
+  | _ -> false
+
+let rec underef ({dt_loc = loc} as dt) = match dt.dt_node with
+  | DTuloc (dt,l) -> Dterm.dterm ?loc Coercion.empty (DTuloc (underef dt, l))
+  | DTattr (dt,a) -> Dterm.dterm ?loc Coercion.empty (DTattr (underef dt, a))
+  | DTcast (dt,_) -> underef dt (* already unified *)
+  | DTapp (ls, [dt]) when ls_equal ls ls_ref_proj && to_deref dt.dt_node -> dt
+  | _ -> raise Not_found
+
+let undereference dt = try underef dt with Not_found ->
+  Loc.errorm ?loc:dt.dt_loc "not a reference variable"
+
+let to_underef ls el e =
+  let rec down al el = match al, el with
+    | (_::al), (_::el) -> down al el
+    | a::_, [] when vs_dref a.pv_vs ->
+        (try underef e with Not_found -> e)
+    | _::_, [] -> e
+    | _ -> assert false in
+  match Expr.restore_rs ls with
+  | rs -> down rs.rs_cty.cty_args el
+  | exception Not_found -> e
+
+let dt_app ls el =
+  let rec apply al l el = match l, el with
+    | ({ty_node = Tyapp (ts,_)}::l), (e::el)
+      when ts_equal ts Pmodule.ts_ref ->
+        (* ls may be a let-function with ref params *)
+        apply (to_underef ls al e::al) l el
+    | (_::l), (e::el) -> apply (e::al) l el
+    | _ -> DTapp (ls, List.rev_append al el) in
+  apply [] ls.ls_args el
+
 (* track the use of labels *)
 let at_uses = Hstr.create 5
 
@@ -227,20 +277,35 @@ let rec dterm ns km crcmap gvars at denv {term_desc = desc; term_loc = loc} =
       DTfapp (Dterm.dterm crcmap ~loc e1, e2)) e el
   in
   let rec apply_ls loc ls al l el = match l, el with
+    | ({ty_node = Tyapp (ts,_)}::l), ((loc_e,e)::el)
+      when ts_equal ts Pmodule.ts_ref ->
+        (* ls may be a let-function with ref params *)
+        apply_ls loc ls ((loc_e, to_underef ls al e)::al) l el
     | (_::l), (e::el) -> apply_ls loc ls (e::al) l el
     | [], _ -> func_app (DTapp (ls, List.rev_map snd al)) el
     | _, [] -> func_app (mk_closure crcmap loc ls) (List.rev_append al el)
   in
   let qualid_app q el = match gvars at q with
     | Some v ->
-        begin match at with
-        | Some l -> (* check for impact *)
-            let u = Opt.get (gvars None q) in
-            if not (pv_equal v u) then
-              Hstr.replace at_uses l true
-        | None -> ()
-        end;
-        func_app (DTgvar v.pv_vs) el
+        let attrs = match at with
+          | Some l -> (* check for impact *)
+              let u = Opt.get (gvars None q) in
+              if pv_equal v u then Sattr.empty else begin
+                let attr = create_attribute ("at:" ^ l) in
+                Hstr.replace at_uses l true;
+                Sattr.singleton attr
+              end
+          | None -> Sattr.empty
+        in
+        let e = DTgvar v.pv_vs in
+        let e = if Sattr.is_empty attrs then e else
+          DTattr (Dterm.dterm crcmap ~loc e, attrs) in
+        if vs_dref v.pv_vs then
+          let e = Dterm.dterm crcmap ~loc e in
+          let loc = qloc q and ls = Pmodule.ls_ref_proj in
+          apply_ls loc ls [] ls.ls_args ((loc, e)::el)
+        else
+          func_app e el
     | None ->
         let ls = find_lsymbol_ns ns q in
         apply_ls (qloc q) ls [] ls.ls_args el
@@ -266,7 +331,7 @@ let rec dterm ns km crcmap gvars at denv {term_desc = desc; term_loc = loc} =
       qualid_app q []
   | Ptree.Tidapp (q, tl) ->
       let tl = List.map (dterm ns km crcmap gvars at denv) tl in
-      DTapp (find_lsymbol_ns ns q, tl)
+      dt_app (find_lsymbol_ns ns q) tl
   | Ptree.Tapply (e1, e2) ->
       unfold_app e1 (dterm ns km crcmap gvars at denv e2) []
   | Ptree.Ttuple tl ->
@@ -278,9 +343,9 @@ let rec dterm ns km crcmap gvars at denv {term_desc = desc; term_loc = loc} =
         if op.id_str = Ident.op_neq then
           let op = { op with id_str = Ident.op_equ } in
           let ls = find_lsymbol_ns ns (Qident op) in
-          DTnot (Dterm.dterm crcmap ~loc (DTapp (ls, [de1;de2])))
+          DTnot (Dterm.dterm crcmap ~loc (dt_app ls [de1;de2]))
         else
-          DTapp (find_lsymbol_ns ns (Qident op), [de1;de2]) in
+          dt_app (find_lsymbol_ns ns (Qident op)) [de1;de2] in
       let rec chain loc de1 op1 = function
         | { term_desc = Ptree.Tinfix (e2, op2, e3); term_loc = loc23 } ->
             let de2 = dterm ns km crcmap gvars at denv e2 in
@@ -375,6 +440,10 @@ let rec dterm ns km crcmap gvars at denv {term_desc = desc; term_loc = loc} =
   | Ptree.Tscope (q, e1) ->
       let ns = import_namespace ns (string_list_of_qualid q) in
       DTattr (dterm ns km crcmap gvars at denv e1, Sattr.empty)
+  | Ptree.Tasref q ->
+      let e1 = { term_desc = Tident q; term_loc = loc } in
+      let d1 = dterm ns km crcmap gvars at denv e1 in
+      DTattr (undereference d1, Sattr.empty)
   | Ptree.Tattr (ATpos uloc, e1) ->
       DTuloc (dterm ns km crcmap gvars at denv e1, uloc)
   | Ptree.Tattr (ATstr attr, e1) ->
@@ -385,7 +454,6 @@ let rec dterm ns km crcmap gvars at denv {term_desc = desc; term_loc = loc} =
       let d1 = dterm ns km crcmap gvars at denv e1 in
       DTcast (d1, dty_of_pty ns pty))
 
-
 let no_gvars at q = match at with
   | Some _ -> Loc.errorm ~loc:(qloc q)
       "`at' and `old' can only be used in program annotations"
@@ -408,7 +476,6 @@ let ty_of_pty tuc = ty_of_pty (get_namespace tuc)
 
 let get_namespace muc = List.hd muc.Pmodule.muc_import
 
-
 let dterm muc =
   let uc = muc.muc_theory in
   dterm (Theory.get_namespace uc) uc.uc_known uc.uc_crcmap
@@ -438,6 +505,8 @@ let rec ity_of_pty muc = function
       Loc.try3 ~loc:(qloc q) ity_app s tyl []
   | PTtuple tyl ->
       ity_tuple (List.map (ity_of_pty muc) tyl)
+  | PTref tyl ->
+      ity_app its_ref (List.map (ity_of_pty muc) tyl) []
   | PTarrow (ty1, ty2) ->
       ity_func (ity_of_pty muc ty1) (ity_of_pty muc ty2)
   | PTpure ty ->
@@ -515,7 +584,7 @@ let rec dpattern muc gh { pat_desc = desc; pat_loc = loc } =
     | Ptree.Pparen _ | Ptree.Pscope _
     | Ptree.Pghost _ -> assert false (* never *)
     | Ptree.Pwild -> DPwild
-    | Ptree.Pvar x -> DPvar (create_user_id x, gh)
+    | Ptree.Pvar x -> DPvar (create_user_prog_id x, gh)
     | Ptree.Papp (q,pl) ->
         DPapp (find_constructor muc q, List.map (dpattern muc gh) pl)
     | Ptree.Prec fl ->
@@ -526,9 +595,12 @@ let rec dpattern muc gh { pat_desc = desc; pat_loc = loc } =
         DPapp (cs,fl)
     | Ptree.Ptuple pl ->
         DPapp (rs_tuple (List.length pl), List.map (dpattern muc gh) pl)
-    | Ptree.Pcast (p,pty) -> DPcast (dpattern muc gh p, dity_of_pty muc pty)
-    | Ptree.Pas (p,x,g) -> DPas (dpattern muc gh p, create_user_id x, gh || g)
-    | Ptree.Por (p,q) -> DPor (dpattern muc gh p, dpattern muc gh q))
+    | Ptree.Pcast (p,pty) ->
+        DPcast (dpattern muc gh p, dity_of_pty muc pty)
+    | Ptree.Pas (p,x,g) ->
+        DPas (dpattern muc gh p, create_user_prog_id x, gh || g)
+    | Ptree.Por (p,q) ->
+        DPor (dpattern muc gh p, dpattern muc gh q))
 
 let dpattern muc pat = dpattern muc false pat
 
@@ -663,18 +735,19 @@ let dinvariant muc f lvm _xsm old = dpre muc f lvm old
 (* abstract values *)
 
 let dparam muc (_,id,gh,pty) =
-  Opt.map create_user_id id, gh, dity_of_pty muc pty
+  Opt.map create_user_prog_id id, gh, dity_of_pty muc pty
 
 let dbinder muc (_,id,gh,opt) =
-  Opt.map create_user_id id, gh, dity_of_opt muc opt
+  Opt.map create_user_prog_id id, gh, dity_of_opt muc opt
 
 (* expressions *)
 
 let is_reusable de = match de.de_node with
   | DEvar _ | DEsym _ -> true | _ -> false
 
-let mk_var n de =
-  Dexpr.dexpr ?loc:de.de_loc (DEvar (n, de.de_dvty))
+let mk_var n { de_dvty = (argl, _ as dvty); de_loc = loc } =
+  let dref = List.map Util.ffalse argl, false in
+  Dexpr.dexpr ?loc (DEvar (n, dvty, dref))
 
 let mk_let ~loc n de node =
   let de1 = Dexpr.dexpr ~loc node in
@@ -689,6 +762,10 @@ let local_kind = function
   | RKfunc | RKpred -> RKlocal
   | k -> k
 
+let undereference ({de_loc = loc} as de) =
+  try Dexpr.undereference de with Not_found ->
+    Loc.errorm ?loc "not a reference variable"
+
 let rec eff_dterm muc denv {term_desc = desc; term_loc = loc} =
   let expr_app loc e el =
     List.fold_left (fun e1 e2 ->
@@ -717,6 +794,10 @@ let rec eff_dterm muc denv {term_desc = desc; term_loc = loc} =
       let muc = open_scope muc "dummy" in
       let muc = import_scope muc (string_list_of_qualid q) in
       DEattr (eff_dterm muc denv e1, Sattr.empty)
+  | Ptree.Tasref q ->
+      let e1 = { term_desc = Tident q; term_loc = loc } in
+      let d1 = eff_dterm muc denv e1 in
+      DEattr (undereference d1, Sattr.empty)
   | Ptree.Tattr (ATpos uloc, e1) ->
       DEuloc (eff_dterm muc denv e1, uloc)
   | Ptree.Tattr (ATstr attr, e1) ->
@@ -809,6 +890,8 @@ let rec dexpr muc denv {expr_desc = desc; expr_loc = loc} =
       let dty = if Mts.is_empty muc.muc_theory.uc_floats
                 then dity_real else dity_fresh () in
       DEconst(c, dty)
+  | Ptree.Eref ->
+      DEsym (RS rs_ref)
   | Ptree.Erecord fl ->
       let ls_of_rs rs = match rs.rs_logic with
         | RLls ls -> ls | _ -> assert false in
@@ -832,7 +915,7 @@ let rec dexpr muc denv {expr_desc = desc; expr_loc = loc} =
   | Ptree.Elet (id, gh, kind, e1, e2) ->
       let e1 = update_any kind e1 in
       let kind = local_kind kind in
-      let ld = create_user_id id, gh, kind, dexpr muc denv e1 in
+      let ld = create_user_prog_id id, gh, kind, dexpr muc denv e1 in
       DElet (ld, dexpr muc (denv_add_let denv ld) e2)
   | Ptree.Erec (fdl, e1) ->
       let update_kind (id, gh, k, bl, pty, msk, sp, e) =
@@ -913,12 +996,15 @@ let rec dexpr muc denv {expr_desc = desc; expr_loc = loc} =
       let efrom = dexpr muc denv efrom in
       let eto = dexpr muc denv eto in
       let inv = dinvariant muc inv in
-      let id = create_user_id id in
+      let id = create_user_prog_id id in
       let denv = denv_add_for_index denv id efrom.de_dvty in
       DEfor (id, efrom, dir, eto, inv, dexpr muc denv e1)
   | Ptree.Eassign asl ->
       let mk_assign (e1,q,e2) =
-        dexpr muc denv e1, find_record_field muc q, dexpr muc denv e2 in
+        let f = match q with
+          | Some q -> find_record_field muc q
+          | None -> rs_ref_proj in
+        dexpr muc denv e1, f, dexpr muc denv e2 in
       DEassign (List.map mk_assign asl)
   | Ptree.Eraise (q, e1) ->
       let xs = find_dxsymbol q in
@@ -960,6 +1046,10 @@ let rec dexpr muc denv {expr_desc = desc; expr_loc = loc} =
       let muc = open_scope muc "dummy" in
       let muc = import_scope muc (string_list_of_qualid q) in
       DEattr (dexpr muc denv e1, Sattr.empty)
+  | Ptree.Easref q ->
+      let e1 = { expr_desc = Eident q; expr_loc = loc } in
+      let d1 = dexpr muc denv e1 in
+      DEattr (undereference d1, Sattr.empty)
   | Ptree.Eattr (ATpos uloc, e1) ->
       DEuloc (dexpr muc denv e1, uloc)
   | Ptree.Eattr (ATstr attr, e1) ->
@@ -979,7 +1069,7 @@ and drec_defn muc denv fdl =
       let denv = denv_add_args denv bl in
       let dv = dvariant muc sp.sp_variant in
       dspec muc sp, dv, dexpr muc denv e in
-    create_user_id id, gh, kind, bl, dity, msk, pre in
+    create_user_prog_id id, gh, kind, bl, dity, msk, pre in
   Dexpr.drec_defn denv (List.map prep fdl)
 
 
@@ -1153,6 +1243,7 @@ let add_types muc tdl =
                 find_itysymbol muc q in
           Loc.try3 ~loc:(qloc q) ity_app s (List.map (down muc) tyl) []
       | PTtuple tyl -> ity_tuple (List.map (down muc) tyl)
+      | PTref tyl -> ity_app its_ref (List.map (down muc) tyl) []
       | PTarrow (ty1,ty2) -> ity_func (down muc ty1) (down muc ty2)
       | PTpure ty -> ity_purify (down muc ty)
       | PTscope (q,ty) ->
@@ -1367,7 +1458,7 @@ let add_decl muc env file d =
       add_meta muc (lookup_meta id.id_str) (List.map convert al)
   | Ptree.Dlet (id, gh, kind, e) ->
       let e = update_any kind e in
-      let ld = create_user_id id, gh, kind, dexpr muc denv_empty e in
+      let ld = create_user_prog_id id, gh, kind, dexpr muc denv_empty e in
       add_pdecl ~vc muc (create_let_decl (let_defn ~keep_loc:true ld))
   | Ptree.Drec fdl ->
       let _, rd = drec_defn muc denv_empty fdl in
