@@ -1,7 +1,7 @@
 (********************************************************************)
 (*                                                                  *)
 (*  The Why3 Verification Platform   /   The Why3 Development Team  *)
-(*  Copyright 2010-2018   --   Inria - CNRS - Paris-Sud University  *)
+(*  Copyright 2010-2019   --   Inria - CNRS - Paris-Sud University  *)
 (*                                                                  *)
 (*  This software is distributed under the terms of the GNU Lesser  *)
 (*  General Public License version 2.1, with the special exception  *)
@@ -28,11 +28,6 @@ let debug = Debug.register_info_flag "smtv2_printer"
 let debug_incremental = Debug.register_info_flag "force_incremental"
     ~desc:"Force@ incremental@ mode@ for@ smtv2@ provers"
 
-(* Meta to tag projection functions *)
-let meta_projection = Theory.register_meta "model_projection" [Theory.MTlsymbol]
-  ~desc:"Declares@ the@ projection."
-
-
 (** SMTLIB tokens taken from CVC4: src/parser/smt2/{Smt2.g,smt2.cpp} *)
 let ident_printer () =
   let bls =
@@ -117,6 +112,9 @@ let ident_printer () =
      (* From Z3 *)
       "map"; "bv"; "default";
       "difference";
+
+     (* Counterexamples specific keywords *)
+      "model";
       ]
   in
   let san = sanitizer char_to_alpha char_to_alnumus in
@@ -126,20 +124,25 @@ type version = V20 | V26
 
 type info = {
   info_syn        : syntax_map;
-  info_converters : converter_map;
   info_rliteral   : syntax_map;
   mutable info_model : S.t;
   mutable info_in_goal : bool;
   info_vc_term : vc_term_info;
   info_printer : ident_printer;
-  mutable list_projs : Sstr.t;
+  mutable list_projs : Ident.ident Mstr.t;
+  mutable list_field_def: Ident.ident Mstr.t;
   info_version : version;
   meta_model_projection : Sls.t;
+  meta_record_def : Sls.t;
   mutable list_records : ((string * string) list) Mstr.t;
+  (* For algebraic type counterexamples: constructors with no arguments can be
+     misunderstood for variables *)
+  mutable noarg_constructors: string list;
   info_cntexample_need_push : bool;
   info_cntexample: bool;
   info_incremental: bool;
   info_set_incremental: bool;
+  mutable info_labels: Sattr.t Mstr.t;
   mutable incr_list: (prsymbol * term) list;
 }
 
@@ -194,8 +197,12 @@ let print_var_list info fmt vsl =
 
 let collect_model_ls info ls =
   if Sls.mem ls info.meta_model_projection then
-    info.list_projs <- Sstr.add (sprintf "%a" (print_ident info) ls.ls_name) info.list_projs;
-  if ls.ls_args = [] && (has_a_model_attr ls.ls_name) then
+    info.list_projs <- Mstr.add (sprintf "%a" (print_ident info) ls.ls_name)
+        ls.ls_name info.list_projs;
+  if Sls.mem ls info.meta_record_def then
+    info.list_field_def <- Mstr.add (sprintf "%a" (print_ident info) ls.ls_name)
+        ls.ls_name info.list_field_def;
+  if ls.ls_args = [] && (relevant_for_counterexample ls.ls_name) then
     let t = t_app ls [] ls.ls_value in
     info.info_model <-
       add_model_element
@@ -221,8 +228,8 @@ let number_format = {
 (** expr *)
 let rec print_term info fmt t =
   debug_print_term "Printing term: " t;
-
-  if Sattr.exists is_model_trace_attr t.t_attrs then
+  if check_for_counterexample t
+  then
     info.info_model <- add_model_element t info.info_model;
 
   check_enter_vc_term t info.info_in_goal info.info_vc_term;
@@ -249,34 +256,28 @@ let rec print_term info fmt t =
       end
   | Tvar v -> print_var info fmt v
   | Tapp (ls, tl) ->
-    (* let's check if a converter applies *)
-    begin try
-      match tl with
-      | [ { t_node = Tconst _} ] ->
-        begin match query_converter info.info_converters ls with
-        | None -> raise Exit
-        | Some s -> syntax_arguments s (print_term info) fmt tl
-        end
-      | _ -> raise Exit
-    with Exit ->
-    (* non converter applies, then ... *)
-    match query_syntax info.info_syn ls.ls_name with
+    begin match query_syntax info.info_syn ls.ls_name with
       | Some s -> syntax_arguments_typed s (print_term info)
         (print_type info) t fmt tl
       | None -> begin match tl with (* for cvc3 wich doesn't accept (toto ) *)
           | [] ->
-	    let vc_term_info = info.info_vc_term in
-	    if vc_term_info.vc_inside then begin
-	      match vc_term_info.vc_loc with
-	      | None -> ()
-	      | Some loc ->
+
+            let str_ls = sprintf "%a" (print_ident info) ls.ls_name in
+            let cur_var = info.info_labels in
+            let new_var = update_info_labels str_ls cur_var t ls in
+            let () = info.info_labels <- new_var in
+            let vc_term_info = info.info_vc_term in
+            if vc_term_info.vc_inside then begin
+              match vc_term_info.vc_loc with
+              | None -> ()
+              | Some loc ->
                 let attrs = (* match vc_term_info.vc_func_name with
                   | None -> *)
                     ls.ls_name.id_attrs
                   (* | Some _ ->
                     model_trace_for_postcondition ~attrs:ls.ls_name.id_attrs info.info_vc_term
                    *)
-in
+                in
 		let _t_check_pos = t_attr_set ~loc attrs t in
 		(* TODO: temporarily disable collecting variables inside the term triggering VC *)
 		(*info.info_model <- add_model_element t_check_pos info.info_model;*)
@@ -287,7 +288,8 @@ in
 	    fprintf fmt "@[(%a@ %a)@]"
 	      (print_ident info) ls.ls_name
               (print_list space (print_term info)) tl
-        end end
+                end
+    end
   | Tlet (t1, tb) ->
       let v, t2 = t_open_bound tb in
       fprintf fmt "@[(let ((%a %a))@ %a)@]" (print_var info) v
@@ -321,7 +323,8 @@ in
 
 and print_fmla info fmt f =
   debug_print_term "Printing formula: " f;
-  if Sattr.exists is_model_trace_attr f.t_attrs then
+  if check_for_counterexample f
+  then
     info.info_model <- add_model_element f info.info_model;
 
   check_enter_vc_term f info.info_in_goal info.info_vc_term;
@@ -333,7 +336,8 @@ and print_fmla info fmt f =
       | Some s -> syntax_arguments_typed s (print_term info)
         (print_type info) f fmt tl
       | None -> begin match tl with (* for cvc3 wich doesn't accept (toto ) *)
-          | [] -> print_ident info fmt ls.ls_name
+          | [] ->
+              print_ident info fmt ls.ls_name
           | _ -> fprintf fmt "(%a@ %a)"
               (print_ident info) ls.ls_name
               (print_list space (print_term info)) tl
@@ -499,7 +503,7 @@ let print_info_model info =
       let model_map =
 	S.fold (fun f acc ->
           let s = asprintf "%a" (print_fmla info) f in
-	  Mstr.add s f acc)
+          Mstr.add s f acc)
 	info_model
 	Mstr.empty in
 
@@ -579,13 +583,20 @@ let print_prop_decl vc_loc args info fmt k pr f = match k with
                                 vc_term_loc = vc_loc;
                                 queried_terms = model_list;
                                 list_projections = info.list_projs;
-                                Printer.list_records = info.list_records}
+                                list_fields = info.list_field_def;
+                                Printer.list_records = info.list_records;
+                                noarg_constructors = info.noarg_constructors;
+                                set_str = info.info_labels;
+                              }
   | Plemma -> assert false
 
 let print_constructor_decl info fmt (ls,args) =
   let field_names =
     (match args with
-    | [] -> fprintf fmt "(%a)" (print_ident info) ls.ls_name; []
+    | [] -> fprintf fmt "(%a)" (print_ident info) ls.ls_name;
+        let cons_name = sprintf "%a" (print_ident info) ls.ls_name in
+        info.noarg_constructors <- cons_name :: info.noarg_constructors;
+        []
     | _ ->
         fprintf fmt "@[(%a@ " (print_ident info) ls.ls_name;
         let field_names, _ =
@@ -700,22 +711,25 @@ let print_task version args ?old:_ fmt task =
   let vc_info = {vc_inside = false; vc_loc = None; vc_func_name = None} in
   let info = {
     info_syn = Discriminate.get_syntax_map task;
-    info_converters = Printer.get_converter_map task;
     info_rliteral = Printer.get_rliteral_map task;
     info_model = S.empty;
     info_in_goal = false;
     info_vc_term = vc_info;
     info_printer = ident_printer ();
-    list_projs = Sstr.empty;
+    list_projs = Mstr.empty;
+    list_field_def = Mstr.empty;
     info_version = version;
-    meta_model_projection = Task.on_tagged_ls meta_projection task;
+    meta_model_projection = Task.on_tagged_ls Theory.meta_projection task;
+    meta_record_def = Task.on_tagged_ls Theory.meta_record task;
     list_records = Mstr.empty;
+    noarg_constructors = [];
     info_cntexample_need_push = need_push;
     info_cntexample = cntexample;
     info_incremental = incremental;
     (* info_set_incremental add the incremental option to the header. It is not
        needed for some provers
     *)
+    info_labels = Mstr.empty;
     info_set_incremental = not need_push && incremental;
     incr_list = [];
     }
