@@ -1,7 +1,7 @@
 (********************************************************************)
 (*                                                                  *)
 (*  The Why3 Verification Platform   /   The Why3 Development Team  *)
-(*  Copyright 2010-2018   --   Inria - CNRS - Paris-Sud University  *)
+(*  Copyright 2010-2019   --   Inria - CNRS - Paris-Sud University  *)
 (*                                                                  *)
 (*  This software is distributed under the terms of the GNU Lesser  *)
 (*  General Public License version 2.1, with the special exception  *)
@@ -174,9 +174,9 @@ val reload_files : controller -> shape_version:int option -> bool * bool
  *)
 
 val add_file : controller -> ?format:Env.fformat -> string -> unit
-(** [add_fil cont ?fmt fname] parses the source file
+(** [add_file cont ?fmt fname] parses the source file
     [fname] and add the resulting theories to the session of [cont].
-    parsing or typing errors are raised inside exception Errors_list
+    parsing or typing errors are raised inside exception [Errors_list]
  *)
 
 val remove_subtree: notification:notifier -> removed:notifier ->
@@ -248,6 +248,7 @@ val prepare_edition :
     session directory, and [res] is the former result if any. *)
 
 exception TransAlreadyExists of string * string
+exception GoalNodeDetached of proofNodeID
 
 val schedule_transformation :
   controller ->
