@@ -1,7 +1,7 @@
 (********************************************************************)
 (*                                                                  *)
 (*  The Why3 Verification Platform   /   The Why3 Development Team  *)
-(*  Copyright 2010-2018   --   Inria - CNRS - Paris-Sud University  *)
+(*  Copyright 2010-2019   --   Inria - CNRS - Paris-Sud University  *)
 (*                                                                  *)
 (*  This software is distributed under the terms of the GNU Lesser  *)
 (*  General Public License version 2.1, with the special exception  *)
@@ -122,7 +122,13 @@ let () = try
       (List.sort sort_pair (Trans.list_transforms ()));
     printf "@[<hov 2>Known splitting transformations:@\n%a@]@\n@."
       (Pp.print_list Pp.newline2 print_trans_desc)
-      (List.sort sort_pair (Trans.list_transforms_l ()))
+      (List.sort sort_pair (Trans.list_transforms_l ()));
+    let list_transform_with_arg =
+      Trans.list_transforms_with_args () @ Trans.list_transforms_with_args_l ()
+    in
+    printf "@[<hov 2>Known transformations with arguments:@\n%a@]@\n@."
+      (Pp.print_list Pp.newline2 print_trans_desc)
+      (List.sort sort_pair list_transform_with_arg)
   end;
   if !opt_list_printers then begin
     opt_list := true;
