@@ -1,7 +1,7 @@
 (********************************************************************)
 (*                                                                  *)
 (*  The Why3 Verification Platform   /   The Why3 Development Team  *)
-(*  Copyright 2010-2018   --   Inria - CNRS - Paris-Sud University  *)
+(*  Copyright 2010-2019   --   Inria - CNRS - Paris-Sud University  *)
 (*                                                                  *)
 (*  This software is distributed under the terms of the GNU Lesser  *)
 (*  General Public License version 2.1, with the special exception  *)
@@ -190,6 +190,15 @@ let () = Trans.register_transform "introduce_premises" introduce_premises
   ~desc:"Introduce@ universal@ quantification@ and@ hypothesis@ in@ the@ \
          goal@ into@ constant@ symbol@ and@ axioms."
 
+(* In this file t_replace is used to substitute vsymbol with lsymbols. This is
+   done in [set_vs]; but in cases where the attribute is directly on the lsymbol
+   term application (Tapp), the substitution may not work resulting in an error
+   of the transformation. That's why we check for equality modulo attributes and
+   then copy attributes back on the term again.
+*)
+let rec t_replace t1 t2 t =
+  if t_equal_nt_na t t1 then t_attr_copy t t2 else t_map (t_replace t1 t2) t
+
 let rec generalize hd =
   match hd.Task.task_decl.Theory.td_node with
   | Theory.Decl {d_node = Dprop (Pgoal,pr,f)} ->
@@ -280,11 +289,11 @@ let () = Trans.register_transform
 
 let subst_filter ls =
   Sattr.mem intro_attr ls.ls_name.id_attrs &&
-  not (Ident.has_a_model_attr ls.ls_name)
+  not (relevant_for_counterexample ls.ls_name)
 
 let simplify_intros =
   Trans.compose introduce_premises
-                (Subst.subst_filtered subst_filter)
+                (Subst.subst_filtered ~subst_proxy:false subst_filter)
 
 let split_vc =
   Trans.compose_l
