@@ -1,7 +1,7 @@
 (********************************************************************)
 (*                                                                  *)
 (*  The Why3 Verification Platform   /   The Why3 Development Team  *)
-(*  Copyright 2010-2018   --   Inria - CNRS - Paris-Sud University  *)
+(*  Copyright 2010-2019   --   Inria - CNRS - Paris-Sud University  *)
 (*                                                                  *)
 (*  This software is distributed under the terms of the GNU Lesser  *)
 (*  General Public License version 2.1, with the special exception  *)
@@ -33,8 +33,10 @@ let prepare_for_counterexmp2 env task =
     (* Counter-example will be queried, prepare the task *)
     Debug.dprintf debug "Get ce@.";
     let comp_trans = Trans.seq [
+      Introduction.simplify_intros;
       Intro_vc_vars_counterexmp.intro_vc_vars_counterexmp;
-      Intro_projections_counterexmp.intro_projections_counterexmp env ] in
+      Intro_projections_counterexmp.intro_projections_counterexmp env
+    ] in
     Trans.apply comp_trans task
   end
 
