@@ -1,7 +1,7 @@
 (********************************************************************)
 (*                                                                  *)
 (*  The Why3 Verification Platform   /   The Why3 Development Team  *)
-(*  Copyright 2010-2018   --   Inria - CNRS - Paris-Sud University  *)
+(*  Copyright 2010-2019   --   Inria - CNRS - Paris-Sud University  *)
 (*                                                                  *)
 (*  This software is distributed under the terms of the GNU Lesser  *)
 (*  General Public License version 2.1, with the special exception  *)
@@ -18,7 +18,8 @@ open Why3
 open Format
 open Worker_proto
 
-
+module Sys_js = Js_of_ocaml.Sys_js
+module Worker = Js_of_ocaml.Worker
 
 let () = log_time ("Initialising why3 worker: start ")
 
@@ -199,11 +200,7 @@ module Task =
 
     let set_status id st =
       let rec loop id st acc =
-        match
-          try
-            Some (get_info id)
-          with Not_found -> None
-        with
+        match get_info_opt id with
         | Some info when info.status <> st ->
 	   info.status <- st;
            let acc = (UpdateStatus (st, id)) :: acc in
@@ -269,10 +266,8 @@ let why3_split id =
 	[], _ -> ()
       | [ child ], `Task(orig) when Why3.Task.task_equal child orig -> ()
       | subtasks, _ ->
-	 t.subtasks <- List.fold_left (fun acc t ->
-				       let tid = register_task id t in
-				       why3_prove tid;
-				       tid :: acc) [] subtasks
+          t.subtasks <- List.map (fun t -> register_task id t) subtasks;
+          List.iter why3_prove t.subtasks
     end
   | _ -> ()
 
