@@ -1,7 +1,7 @@
 (********************************************************************)
 (*                                                                  *)
 (*  The Why3 Verification Platform   /   The Why3 Development Team  *)
-(*  Copyright 2010-2018   --   Inria - CNRS - Paris-Sud University  *)
+(*  Copyright 2010-2019   --   Inria - CNRS - Paris-Sud University  *)
 (*                                                                  *)
 (*  This software is distributed under the terms of the GNU Lesser  *)
 (*  General Public License version 2.1, with the special exception  *)
@@ -9,6 +9,8 @@
 (*                                                                  *)
 (********************************************************************)
 
+open Js_of_ocaml
+
 type id = string
 type loc = int * int * int * int
 type why3_loc = string * (int * int * int) (* kind, line, column, length *)
@@ -45,6 +47,6 @@ let status_of_result = function
 
 let log s = ignore (Firebug.console ## log (Js.string s))
 let log_time s =
-  let date = jsnew Js.date_now () in
-  let date_str = string_of_float ((date ## getTime ()) /. 1000.) in
+  let date = new%js Js.date_now in
+  let date_str = string_of_float (date ## getTime /. 1000.) in
   log (date_str ^ " : " ^ s)
