@@ -1,7 +1,7 @@
 (********************************************************************)
 (*                                                                  *)
 (*  The Why3 Verification Platform   /   The Why3 Development Team  *)
-(*  Copyright 2010-2018   --   Inria - CNRS - Paris-Sud University  *)
+(*  Copyright 2010-2019   --   Inria - CNRS - Paris-Sud University  *)
 (*                                                                  *)
 (*  This software is distributed under the terms of the GNU Lesser  *)
 (*  General Public License version 2.1, with the special exception  *)
@@ -62,6 +62,7 @@ let convert_record l =
 
 type json =
   | Record of json Mstr.t
+  | Proj of json Mstr.t
   | List of json list
   | String of string
   | Int of int
@@ -72,6 +73,7 @@ type json =
 let rec print_json fmt v =
   match v with
   | Record r -> map_bindings (fun x -> x) print_json fmt (Mstr.bindings r)
+  | Proj p -> map_bindings (fun x -> x) print_json fmt (Mstr.bindings p)
   | List l -> list print_json fmt l
   | String s -> string fmt s
   | Int i -> int fmt i
