@@ -1,7 +1,7 @@
 (********************************************************************)
 (*                                                                  *)
 (*  The Why3 Verification Platform   /   The Why3 Development Team  *)
-(*  Copyright 2010-2018   --   Inria - CNRS - Paris-Sud University  *)
+(*  Copyright 2010-2019   --   Inria - CNRS - Paris-Sud University  *)
 (*                                                                  *)
 (*  This software is distributed under the terms of the GNU Lesser  *)
 (*  General Public License version 2.1, with the special exception  *)
@@ -102,38 +102,25 @@ let rec copy_dir from to_ =
     else copy_file src dst in
   Array.iter copy files
 
+let concat f1 f2 =
+  if Filename.is_relative f2 then Filename.concat f1 f2 else f2
 
-(* return the absolute path of a given file name.
-   this code has been designed to be architecture-independant so
-   be very careful if you modify this *)
-let path_of_file f =
+let system_independent_path_of_file f =
   let rec aux acc f =
-(*
-    Format.printf "aux %s@." f;
-    let _ = read_line () in
-*)
     let d = Filename.dirname f in
     if d = Filename.current_dir_name then
       (* f is relative to the current dir *)
       let b = Filename.basename f in
-      aux (b::acc) (Sys.getcwd ())
+      b::acc
     else if f=d then
       (* we are at the root *)
       acc
     else
       let b = Filename.basename f in
-        if f=b then b::acc else
-          aux (b::acc) d
+      aux (b::acc) d
   in
   aux [] f
 
-(* return the file name of an absolute path *)
-let rec file_of_path l =
-  match l with
-    | [] -> ""
-    | [x] -> x
-    | x::l -> Filename.concat x (file_of_path l)
-
 (*
 let test x = (Filename.dirname x, Filename.basename x)
 
@@ -148,26 +135,15 @@ let p1 = path_of_file "/bin/bash"
 let p1 = path_of_file "../src/f.why"
   *)
 
-                              (*
-let normalize_filename f =
-  let rec aux af acc =
-    match af, acc with
-      | x::rf, _ ->
-        if x = Filename.current_dir_name then
-          aux rf acc
-        else if x = Filename.parent_dir_name then
-          (match acc with
-             | _::racc -> aux rf racc
-             | [] ->
-               (* do not treat currently cases like "../../../a/b/c/d",
-                  that cannot occur if [f] is a full path *)
-                failwith "cannot normalize filename")
-        else
-          aux rf (x::acc)
-      | [], _ -> acc
+let system_dependent_absolute_path dir p =
+  let rec file_of_path l =
+    match l with
+    | [] -> ""
+    | [x] -> x
+    | x::l -> Filename.concat x (file_of_path l)
   in
-  file_of_path (List.rev (aux (path_of_file f) []))
-                               *)
+  let f = file_of_path p in
+  Filename.concat dir f
 
 let relativize_filename base f =
   let rec aux abs ab af =
@@ -188,13 +164,9 @@ let relativize_filename base f =
                    aux2 (x::abs) (Filename.parent_dir_name::rel) rb)
           in aux2 abs af ab
   in
-  file_of_path (aux [] (path_of_file base) (path_of_file f))
+  aux [] (system_independent_path_of_file base) (system_independent_path_of_file f)
+
 
-let absolutize_filename dirname f =
-  if Filename.is_relative f then
-    Filename.concat dirname f
-  else
-    f
 
 (*
 let p1 = relativize_filename "/bin/bash" "src/f.why"
