@@ -1,7 +1,7 @@
 (********************************************************************)
 (*                                                                  *)
 (*  The Why3 Verification Platform   /   The Why3 Development Team  *)
-(*  Copyright 2010-2018   --   Inria - CNRS - Paris-Sud University  *)
+(*  Copyright 2010-2019   --   Inria - CNRS - Paris-Sud University  *)
 (*                                                                  *)
 (*  This software is distributed under the terms of the GNU Lesser  *)
 (*  General Public License version 2.1, with the special exception  *)
@@ -117,6 +117,9 @@ val get_transf_string : Session_itp.session -> Session_itp.transID -> string
 (** [get_transf_string s tr] concatenates the name of transformation [tr]
     in session [s] with its arguments *)
 
+val goal_full_name : Session_itp.session ->  Session_itp.proofNodeID -> string
+(** name of goal, taking account explanation *)
+
 val goal_depth : Session_itp.session -> Session_itp.proofNodeID -> int
 (** [goal_depth s g] returns the depth of the tree under goal
     [g] in session [s] *)
